package inline;

/*
 * PROBLEMS:
 * 1) When updating from File-Prio mode, deleted files are still in list
 * 2) Don't point to all subfolders correctly
 *
 */

import inline.fs.*;
import inline.ui.*;
import inline.sys.*;
import java.util.*;
import javax.microedition.io.*;
import java.io.*;
import inline.ui.le.*;

class PlaylistInvalidator extends Thread
{
    private final static String RMS_PLAYLIST_NAME = "Playlist";
    private final static String PLAYLIST_FILENAMES = "files";
    private final static String PLAYLIST_POSITIONS_OLD = "poss";
    private final static String PLAYLIST_STATUSES = "stat";
    private final static String PLAYLIST_NPOSITIONS = "posn";
    private final static char COMPESSEDPREVDIR = (char)29;
    
    public final static int PLAYLIST_LOAD = 0xAB3626D5; // fancy?
    public final static int PLAYLIST_SAVE = 0x864C4B60;

    private Vector list;
    private InPlayer player;
    private HostCanvas parent;
    private String[] plmfn;
    private long[] plmpos;
    private String[] exts;
    private byte[] extfls;
    private int process;

    PlaylistInvalidator(InPlayer ipl, int pprocess)
    {
	this.setPriority(MAX_PRIORITY);
	player = ipl;
	process = pprocess;
    }
        
    public void setParent(HostCanvas pparent)
    {
	parent = pparent;
    }
    
    public void run()
    {
	try
	{
	    if (process == PLAYLIST_LOAD)
	    {
		invalidate();
	    }
	    else if(process == PLAYLIST_SAVE)
	    {
		storeMemory();  
	    }  
/*#Emulator#*///<editor-fold>
//--	    Thread.sleep(1000);
/*$Emulator$*///</editor-fold>  
	}
	catch(Exception e)
	{
	    Log.fire("PL Invalidate failed: "+e);
	}

	if (parent!=null)
	{
	    parent.selectPreview(this);
	    parent=null;
	}
    }

    public Vector getList()
    {
        return list;
    }
    
    public void setList(Vector vct)
    {
        list = vct;
    }

    public void invalidate()
    {
	initExtensionsList();

	if (list!=null)
	{
	    // if we updating already loaded list
	    storeMemory(false);
	    list = new Vector();
	}
	else
	{
	    list = new Vector();
	    initMemory();

	    if (Base.getOptionIntDefault(Options.PLAYER_MEMORYFIRST,0)==1 && 
		    plmfn != null && plmfn.length>0)
	    {
		createFromMemory();
		
		return;
	    }
	}
	
        if (Provider.valid())
        {
	    String[] dirs = (String[])Base.getOption(Options.PLAYER_DIRECTORIES);
	    byte[] flgs = (byte[])Base.getOption(Options.PLAYER_DIRECTORYFLAGS);
	    
	    FileAccessible fs = Provider.newfs();
	    
	    for(int di=0;di<dirs.length;di++)
	    {
		int scan = 1;
		if (flgs!=null && di<flgs.length) scan = flgs[di];
		
		if (scan==1)
		{
		    // postfix with /
		    String dname = dirs[di];
		    if (!dname.endsWith("/")) dname=dname+"/";
		    
		    scanDir(fs, dname);
		}
	    }
        }
        else
        {
	    list.addElement(new StringElement(Base.getText(0x0211)));
	    list.addElement(new StringElement(Base.getText(0x0212)));
        }
    }
    
    private void initExtensionsList()
    {
	exts = (String[])Base.getOption(Options.PLAYER_EXTENSIONS);
	extfls = new byte[exts.length];
	byte[] lefs = (byte[])Base.getOption(Options.PLAYER_EXTENSIONFLAGS);
	for(int i = 0;i<exts.length;i++)
	{
	    exts[i] = exts[i].toLowerCase();
	    
	    if (lefs!=null && i<lefs.length) 
	    {
		extfls[i] = lefs[i];
	    }
	    else
	    {
		extfls[i] = 1;
	    }
	}
    }

    private void scanDir(FileAccessible fs, String dname)
    {
	String[] flist = {};

	if (fs.open(dname, Connector.READ))
	{
	    flist = fs.list();
	    fs.close();  
	}
	
	if (flist == null) // no files in this dir
	{
	    return;
	}
	
	Base.sortStringArray(flist); // always sort
	
	for(int i = 0; i<flist.length;i++)
	{
	    String subfile = flist[i];

	    fs.open(subfile, Connector.READ);
	    if (!fs.isDirectory())
	    {
		int extp = subfile.lastIndexOf('.');
		if (extp>0)
		{
		    String ext = subfile.substring(extp).toLowerCase();
	    
		    for(int ei = 0;ei<exts.length;ei++)
		    {
			if (extfls[ei]==1 && ext.compareTo(exts[ei])==0)
			{
			    if (isNewFile(subfile))
			    {
				PlaylistElement ple = new PlaylistElement(subfile);
				refreshMemory(ple);
				list.addElement(ple);
			    }
			}  
		    }
		}
	    }
	    else
	    {
		if (Base.getOptionIntDefault(Options.PLAYER_SUBDIRS,0) == 1)
		{
		    scanDir(fs, subfile);
		}
		else
		{
		    Log.fire("Dir "+subfile+" skipped");
		}
	    }
	}
    }
    
    private void initMemory()
    {
	String plmfl = (String)Base.getOption(Options.PLAYER_PLAYLISTMEMORY);
	SerialHashtableFile plmem = new SerialHashtableFile(plmfl);
	plmem.load();
	
	plmfn = (String[])plmem.get(PLAYLIST_FILENAMES);
	uncompressFilenames(plmfn);  
	    
	Object obj = plmem.get(PLAYLIST_POSITIONS_OLD);
	if (obj!=null)
	{
	    loadMemoryOldStyle(obj); 
	}
	else
	{
	    loadMemory(plmem);
	}

	if (plmfn != null && plmpos != null && plmfn.length != plmpos.length)
	{
	    //something went wrong yesterday, lengths are ought to be equal
	    //-> discard results
	    //or do some trick later... (TODO?)

	    Log.fire("plmfn/plmpos pos mismatch");		    
	    
	    plmfn = null;
	    plmpos = null;
	}
    }
    
    private void uncompressFilenames(String[] fns)
    {
	String lastdir = null;
	if (fns==null) return;
	
	for(int i=0;i<fns.length;i++)
	{
	    if (fns[i].length()>1 && 
		fns[i].charAt(0) == COMPESSEDPREVDIR &&
		lastdir != null)
	    {
		fns[i] = lastdir + fns[i].substring(1);
	    }
	    else
	    {
		int slpos = fns[i].lastIndexOf('/');
		if (slpos>0)
		{
		    lastdir = fns[i].substring(0, slpos+1);
		}
	    }
	}
    }

    private void compressFilenames(String[] fns)
    {
	String lastdir = null;
	if (fns==null) return;
		
	for(int i=0;i<fns.length;i++)
	{
	    int slpos = fns[i].lastIndexOf('/');
	    if (slpos>0)
	    {  
		String fndir = fns[i].substring(0, slpos+1);
		if (lastdir!=null && lastdir.compareTo(fndir)==0)
		{
		    fns[i] = COMPESSEDPREVDIR + fns[i].substring(slpos+1);
		}
		else
		{
		    lastdir = fndir;
		}
	    }  
	}
    }
    
    private void loadMemory(SerialHashtableFile plmem)
    {
	byte[] statuses = (byte[])plmem.get(PLAYLIST_STATUSES);
	Long[] partpos = (Long[])plmem.get(PLAYLIST_NPOSITIONS);
	if (statuses!=null)
	{
	    int ppi = 0;
	    plmpos = new long[statuses.length];  
	    for(int i=0;i<statuses.length;i++)
	    {
		plmpos[i] = statuses[i];
		if (plmpos[i]==1 && partpos!=null) // partial
		{
		    if (ppi<partpos.length)
		    {
		        plmpos[i]=partpos[ppi].longValue();
			ppi++;
		    }
	        }
	    }		    
        }
    }

    private void loadMemoryOldStyle(Object obj)
    {
        // deprecated input file
        // delete this in 2007
	if (obj==null) return;
	
        Long buf[] = (Long[])obj;
        plmpos = new long[buf.length];
        for(int i=0;i<buf.length;i++)
        {
	    plmpos[i] = buf[i].longValue();
        }
    }

    private void refreshMemory(PlaylistElement ple)
    {
	String fln = ple.getFilename();
	
	if (plmfn != null && plmpos != null)
	{
	    for(int i=0;i<plmfn.length;i++)
	    {
		if (plmfn[i].compareTo(fln) == 0)
		{
		    ple.setPosition(plmpos[i]);
		    break;
		}
	    }
	}
    }    
    
    public void storeMemory()
    {
	storeMemory(true);
    }

    public void storeMemory(boolean save)
    {
	String plmfl = (String)Base.getOption(Options.PLAYER_PLAYLISTMEMORY);
	SerialHashtableFile plmem = new SerialHashtableFile(plmfl);

	if (list == null) return;
	  
	plmfn = new String[list.size()];  

	int spo = Base.getOptionIntDefault(Options.PLAYER_SAVESTATUSES,3);
	String lastplayed = player.getLastPlayedFilename();
//System.err.println("SPO:"+spo);	    

        plmpos = new long[list.size()];
        byte plmstas[] = new byte[list.size()];
        Vector partials = new Vector();
	   
        for(int i=0;i<list.size();i++)
        {
	    PlaylistElement ple = (PlaylistElement)list.elementAt(i);
	    
	    String compfn = ple.getFilename();
	    plmfn[i] = compfn;
	    
	    if (spo>0)
	    {
		long pos = ple.getPosition();
		plmpos[i] = pos;
		plmstas[i] = (byte)pos;
		if (pos!=0 && pos!=-1)
		{
		    if (spo>2 || (lastplayed !=null && plmfn[i].compareTo(lastplayed)==0))
		    {
		        partials.addElement(new Long(pos));
		        plmstas[i] = 1;
		    }
		    else
		    {
			plmstas[i] = 0;
		    }
		}
	    }
	}

	// compress filenames only before we save it on disk
	// or else proper update will fail due to incorrect names
	if (save) compressFilenames(plmfn);
	plmem.put(PLAYLIST_FILENAMES, plmfn);

	if (spo>0)
	{
	    Long plmnpos[] = new Long[partials.size()];
	    for(int i=0;i<plmnpos.length;i++)
	    {
		plmnpos[i] = (Long)partials.elementAt(i);
	    }

	    plmem.put(PLAYLIST_STATUSES, plmstas);
	    plmem.put(PLAYLIST_NPOSITIONS, plmnpos);
	}
	
	if (save)
	{
	    plmem.save();
	}
    }
    
    private void createFromMemory()
    {
	if (plmfn==null) return;
        for(int i=0;i<plmfn.length;i++)
        {
	    PlaylistElement ple = new PlaylistElement(plmfn[i]);
	    refreshMemory(ple);
	    list.addElement(ple);
        }  
    }
    
    private boolean isNewFile(String fn)
    {
	boolean isnew = true;
	
	if (list!=null)
	{
	    for(int i=0;i<list.size();i++)
	    {
		PlaylistElement ple = (PlaylistElement)list.elementAt(i);
	    
		if (ple.getFilename().compareTo(fn) == 0)
		{
		    isnew = false;
		    break;
		}
	    }
	}
	
	return isnew;
    }
}

