package inline;

import inline.ui.*;
import inline.sys.*;
import java.util.*;
import javax.microedition.lcdui.*;
import javax.microedition.io.*;
import inline.ui.le.*;
import inline.ui.lo.*;
import inline.ui.ce.*;

public class SetupC extends ListFormCanvas 
{
    private ChoiceElement cce[][];
    private ColorRamp fRamp;
    private StringElement seApply;
    private StringElement seColorVsSchemes;
    private StringElement seRandom;
    private Vector vColors;
    private Vector vSchemes;
    private int[][] cschemes;
    
    public SetupC(HostCanvas pprevious)
    {
	super(pprevious, Base.getText(0x0433), Base.getText(0x0504), Base.getText(0x0505));
	
	fRamp = new ColorRamp(this);
	fRamp.drawMarks(true);
	Panel lpan = (Panel)getLayoutManager().lastElement();
	addElement(fRamp, lpan, 50, 0);

	makeControls();
	makeMenu();
	switchList(0);
    }
	
    private void makeControls()
    {
	int ca[][] = {{},{},{}};
	fixColors();
	fRamp.fixColors();
	ca[0] = fRamp.divideColor(fRamp.getForegroundColor());
	ca[1] = fRamp.divideColor(fRamp.getMiddleRampColor());
	ca[2] = fRamp.divideColor(fRamp.getBackgroundColor());
	
	String[] cps = new String[11];
	int[] cis = new int[11];
	String[] colnames = {Base.getText(0x0461), Base.getText(0x0462), Base.getText(0x0463)};
	String[] pntnames = {Base.getText(0x0441), Base.getText(0x0442), Base.getText(0x0443)};
	
	for(int i=0;i<=10;i++)
	{
	    cps[i] = Integer.toString(10*i)+"%";
	    cis[i] = 255*i/10;
	}
	
	vColors = new Vector();
	cce = new ChoiceElement[3][3];
	
	for(int ci=0;ci<3;ci++)
	{
	    vColors.addElement(new FigureElement(pntnames[ci], fRamp.FIGURE_CIRCLE));
	    for(int cii=0;cii<3;cii++)
	    {
		int def = ca[ci][cii];
		cce[ci][cii] = new ChoiceElement(colnames[cii],
					    cps,
					    cis,
					    def);
		vColors.addElement(cce[ci][cii]);
	    }
	}

	vSchemes = new Vector();
	Integer[] cramps = (Integer[])Base.getOption("_cramps");
	if (cramps!=null)
	{
	    int schemesc = cramps.length/3;
	    String[] snames = new String[schemesc];
	    cschemes = new int[schemesc][3];
	    Hashtable ssort = new Hashtable();
	    int snbase = 0x6000;

	    for(int i=0;i<snames.length;i++)
	    {
		snames[i] = Base.getText(snbase+i);
		ssort.put(snames[i], new Integer(i));
	    }
	    
	    Base.sortStringArray(snames);
	    
	    for(int i=0;i<snames.length;i++)
	    {
		int idx = ((Integer)ssort.get(snames[i])).intValue();
		for(int j=0;j<3;j++)
		{
		    cschemes[i][j] = cramps[3*idx+j].intValue();
		}
		vSchemes.addElement(new StringElement(snames[i]));
	    }
	}
	
	list.setFastScroll(false);
    }

    /* mode=
     * 0 - auto 
     * 1 - colors
     * 2 - schemes
     * 3 - invert
     */
    private void switchList(int mode)
    {
	int idx = -1;
	if (mode == 0)
	{
	    if ((idx = oneOfSchemes())>=0)
	    {
		mode = 2;
	    }
	    else 
	    {
		mode = 1;
	    }
	}
	else if (mode == 3)
	{
	    if (list.getLines() == vColors)
	    {
		mode = 2;
	    }
	    else
	    {
		mode = 1;
	    }
	}
	
	if (mode == 1)
	{
	    list.setLines(vColors);
	    seColorVsSchemes.setString(Base.getText(0x0445));
	}
	else
	{
	    list.setLines(vSchemes);
	    seColorVsSchemes.setString(Base.getText(0x0446));
	    if (idx>=0) list.setSelectedIndex(idx);
	}
	
    }
    
    private void makeMenu()
    {
	seApply = new StringElement(Base.getText(0x0444));
	seColorVsSchemes = new StringElement(Base.getText(0x0445));
	seRandom = new StringElement(Base.getText(0x0447));
	
	Vector lst = new Vector();
	lst.addElement(seApply);
	lst.addElement(seColorVsSchemes);
	lst.addElement(seRandom);
	
	setMenu(lst);
    }  
    
    public void selectPreview(Object obj)
    {
	super.selectPreview(obj);
	
	if (obj instanceof ChoiceElement)
	{
	    refreshRamp();  
	}    
	else if (obj == seApply)
	{
	    apply();
	    menuHide();
	}
	else if (obj == seColorVsSchemes)
	{
	    switchList(3);
	    menuHide();
	}
	else if (obj == seRandom)
	{
	    randomize();
	    switchList(1);
	    menuHide();
	    apply();
	}
	else if (list.getLines() == vSchemes && obj instanceof StringElement)
	{
	    selectScheme(list.getSelectedIndex());
	}
    }
    
    private void refreshRamp()
    {
	fRamp.setForegroundColor(getColorFrom(0));
	fRamp.setMiddleRampColor(getColorFrom(1));
	fRamp.setBackgroundColor(getColorFrom(2));
	
	fRamp.repaint();
    }
    
    private int getColorFrom(int ci)
    {
	int col[] = {0,0,0};
	
	for(int cii=0;cii<3;cii++)
	{
	    col[cii] = cce[ci][cii].getSelectedKey();
        }
	
	return fRamp.joinColor(col);
    }
    
    private void setColor(int ci, int value)
    {
	for(int cii=2;cii>=0;cii--)
	{
	    cce[ci][cii].setSelectedKey(value&0xff);
	    value>>=8;
        }
    }

    private void ensureContrast()
    {
	int dif = 0;
	
	for(int cii=0;cii<3;cii++)
	{
	    dif += Math.abs(cce[0][cii].getSelectedKey() - 
				cce[2][cii].getSelectedKey());
	}
	dif = dif/3;
	
	if (dif<=30)
	{
	    // contrast's too low
	    for(int cii=0;cii<3;cii++)
	    {
		cce[0][cii].setSelectedKey(cce[0][cii].getSelectedKey()^255);
	    }
	}
    }
    
    private void storeColors()
    {
	ensureContrast();
	
	Base.putOptionInt(CANVAS_FOREGROUND,getColorFrom(0));
	Base.putOptionInt(CANVAS_MIDDLERAMP,getColorFrom(1));
	Base.putOptionInt(CANVAS_BACKGROUND,getColorFrom(2));

// DUMP
String lll = "{";
for(int ci=0;ci<3;ci++)
{
lll += "0x"+Integer.toHexString(getColorFrom(ci));
if (ci<2) lll += ", ";
}
lll += "}";
Log.fire(lll);

    }
    
    protected void submit()
    {
	storeColors();
	
	super.submit();
    }
    
    
    private void apply()
    {
	Vector elms = getElements();
	
	for(int i = 0;i<elms.size();i++)
        {
	    CanvasElement ce = (CanvasElement)elms.elementAt(i);
            if (ce!=null)
	    {
		ce.setForegroundColor(getColorFrom(0));
		ce.setMiddleRampColor(getColorFrom(1));
		ce.setBackgroundColor(getColorFrom(2));
	    }
	}	
	
	this.setForegroundColor(getColorFrom(0));
	this.setMiddleRampColor(getColorFrom(1));
	this.setBackgroundColor(getColorFrom(2));
	
	repaint();
    }
    
    private int oneOfSchemes()
    {
	int ca[] = {0,0,0};
	
	ca[0] = fRamp.getForegroundColor();
	ca[1] = fRamp.getMiddleRampColor();
	ca[2] = fRamp.getBackgroundColor();
	
	for(int i=0;i<cschemes.length;i++)
	{
	    boolean res = true;
	    for(int j=0;j<3;j++)
	    {
		res = res & (cschemes[i][j] == ca[j]);
	    }
	    if (res) return i;
	}
	return -1;
    }
    
    private void selectScheme(int idx)
    {
	if (idx<cschemes.length)
	{
	    for(int i=0;i<3;i++)
	    {
		setColor(i, cschemes[idx][i]);
	    }
	}
	
	apply();
    }
    
    private void randomize()
    {
	Random rnd = new Random();
	for(int ci=0;ci<3;ci++)
	{
	    for(int cii=0;cii<3;cii++)
	    {
		cce[ci][cii].setSelectedKey(rnd.nextInt()&0xff);
	    }
	}
	ensureContrast();
    }
    
    public void applyRandomScheme()
    {
	Random rnd = new Random();
	selectScheme(Math.abs(rnd.nextInt()) % cschemes.length);
	storeColors();
    }
}




